@@ -37,9 +37,10 @@ sub new {
     my $class = ref($this) || $this;
     my %args = ( @_ );
     my $self = {
-        logger => Log::Log4perl->get_logger
+        logger => Log::Log4perl->get_logger($class),
     };
     bless $self, $class;
+    weaken($self->{logger});
 
     unless (defined $args{cli}) {
         confess __PACKAGE__, ': Missing cli';
@@ -54,7 +55,7 @@ sub new {
         Lim::WARN and $self->{logger}->warn('Unable to initialize module '.$class.': '.$@);
         return;
     }
-    
+
     Lim::OBJ_DEBUG and $self->{logger}->debug('new ', __PACKAGE__, ' ', $self);
     $self;
 }
@@ -62,7 +63,7 @@ sub new {
 sub DESTROY {
     my ($self) = @_;
     Lim::OBJ_DEBUG and $self->{logger}->debug('destroy ', __PACKAGE__, ' ', $self);
-    
+
     $self->Destroy;
 }
 
@@ -94,12 +95,12 @@ sub cli {
 
 sub Prompt {
     my ($self) = @_;
-    
+
     if (ref($self)) {
         $self = ref($self);
     }
     $self =~ s/::[^:]+$//o;
-    
+
     return '/'.lc($self->Name);
 }
 
@@ -109,7 +110,7 @@ sub Prompt {
 
 sub Successful {
     my ($self) = @_;
-    
+
     if (defined $self->{cli}) {
         $self->{cli}->Successful;
     }
@@ -121,7 +122,7 @@ sub Successful {
 
 sub Error {
     my $self = shift;
-    
+
     if (defined $self->{cli}) {
         $self->{cli}->Error(@_);
     }
