@@ -37,11 +37,12 @@ sub new {
     my $this = shift;
     my $class = ref($this) || $this;
     my $self = {
-        logger => Log::Log4perl->get_logger,
+        logger => Log::Log4perl->get_logger($class),
         maps => []
     };
     bless $self, $class;
-    
+    weaken($self->{logger});
+
     Lim::OBJ_DEBUG and $self->{logger}->debug('new ', __PACKAGE__, ' ', $self);
     $self;
 }
@@ -72,16 +73,17 @@ sub add {
     else {
         $call = '';
     }
-    
-    if (exists $_MAP_CACHE_CODE{$map} and defined $_MAP_CACHE_CODE{$map}) {
-        push(@{$self->{maps}}, $_MAP_CACHE_CODE{$map});
+
+    my $map_key = $map.' '.$call;
+    if (exists $_MAP_CACHE_CODE{$map_key} and defined $_MAP_CACHE_CODE{$map_key}) {
+        push(@{$self->{maps}}, $_MAP_CACHE_CODE{$map_key});
         return $call;
     }
-    
+
     #
     # Validate and pull out parts of the map used to generate regexp and code
     #
-    
+
     foreach my $map_part (split(/\//o, $map)) {
         if ($map_part =~ /^\w+$/o) {
             push(@regexps, $map_part);
@@ -96,7 +98,7 @@ sub add {
             return;
         }
     }
-    
+
     #
     # Validate the regexp made from the map by compiling it with qr
     #
@@ -109,18 +111,20 @@ sub add {
         Lim::DEBUG and $self->{logger}->debug('Regexp compilation of map "', $map, '" failed: ', $@);
         return;
     }
-    
+
     #
     # Generate the code that checked given URI with generated regexp and adds
     # data gotten by the regexp to the data structure defined by the map
     #
-    
+
+    $code = '';
+
     if ($predata) {
         foreach my $predata_variable (split(/,/o, $predata)) {
             if ($predata_variable =~ /^([^=]+)=(.+)$/o) {
                 my ($variable, $value) = ($1, $2);
 
-                $code .= '$data->{'.join('}->{', split(/\./o, $variable)).'} = \''.$value.'\'';
+                $code .= '$data->{'.join('}->{', split(/\./o, $variable)).'} = \''.$value.'\';';
             }
             else {
                 Lim::DEBUG and $self->{logger}->debug('Predata of map "', $map, '" invalid');
@@ -131,47 +135,44 @@ sub add {
     }
 
     if (scalar @variables) {
-        $code = 'my (';
-    
+        $code .= 'my (';
+
         $n = 1;
         while ($n <= scalar @variables) {
             $code .= '$v'.$n.($n != scalar @variables ? ',' : '');
             $n++;
         }
-        
+
         $code .= ')=(';
-    
+
         $n = 1;
         while ($n <= scalar @variables) {
             $code .= '$'.$n.($n != scalar @variables ? ',' : '');
             $n++;
         }
-        
+
         $code .= ');';
-    
+
         $n = 1;
         foreach my $variable (@variables) {
             $code .= '$data->{'.join('}->{', split(/\./o, $variable)).'} = $v'.($n++).';';
         }
     }
-    else {
-        $code = '';
-    }
 
     #
     # Create the subroutine from the generated code
     #
-    
+
     eval '$code = sub { my ($uri, $data)=@_; if($uri =~ /'.$regexp.'/o) { '.$code.' return \''.$call.'\';} return; };';
     if ($@) {
         Lim::DEBUG and $self->{logger}->debug('Code generation of map "', $map, '" failed: ', $@);
         return;
     }
-    
+
     #
     # Verify code by calling it in eval
     #
-    
+
     eval {
         $code->('', {});
     };
@@ -179,13 +180,13 @@ sub add {
         Lim::DEBUG and $self->{logger}->debug('Verify code of map "', $map, '" failed: ', $@);
         return;
     }
-    
+
     #
     # Store the generated subroutine and return success
     #
 
-    $_MAP_CACHE_CODE{$map} = $code;
-    weaken($_MAP_CACHE_CODE{$map});
+    $_MAP_CACHE_CODE{$map_key} = $code;
+    weaken($_MAP_CACHE_CODE{$map_key});
     push(@{$self->{maps}}, $code);
     return $call;
 }
@@ -196,11 +197,11 @@ sub add {
 
 sub process {
     my ($self, $uri, $data) = @_;
-    
+
     unless (ref($data) eq 'HASH') {
         confess '$data parameter is not a hash';
     }
-    
+
     foreach my $map (@{$self->{maps}}) {
         if (defined (my $ret = $map->($uri, $data))) {
             return $ret;
